// ****************************************************************** 
//   TInTex v1.0                                                      
//   TInTex is a string holder designed to hold text inside the exe   
//   file for use during run time.  There are other methods of        
//   achieving a similar result but TInTex is simple to use and       
//   understand, small and does a little more than just hold strings! 
//   Text is held in a strings object and can be simply a single line 
//   or a whole text file.                                            
//   TInTex.Strings has all the methods of TStrings so can be to some 
//   extent searched, read from and written to and the strings        
//   manipulated or deleted. See TStrings help for more information.  
//   Text can be broken into sections, each identified by a heading   
//   in the format "[heading]".  Each section can be text or in the   
//   same format as an INI file.  Each section can be individually    
//   retrieved; useful for loading text into boxes or lists and       
//   messages. You can also retrieve a single value, section values or
//   section names if in the format Name=Value and section headings.  
//   TInTex can contain a combination of both formats but each section
//   must have a heading.                                             
//   Text is loaded at design time using the string List Editor and   
//   saved with the exe.                                              
//   You can add strings at run time but they will not be saved when  
//   the exe closes nor will any manipulation of strings.  You can    
//   save the contents to a file or stream and load from a file or    
//   stream .....  and more, try it.                                  
//   This code is FREEWARE and you can do with it what you will.
//   If you have any suggestions to improve or enhance the code,
//   please let me know.
//
//   FileName: Intex.pas (TInTex v1.0)                                
//   Author: Brian Shepherd                                           
//   Date: 18/07/99                                                   
//   email: bshepherd@mail.com                                        
//                                                                    
// ******************************************************************

UNIT InTex;

INTERFACE

USES WinTypes, WinProcs, SysUtils,Classes;

TYPE

  ENotFound = CLASS(Exception);

  TInTex = CLASS(TComponent)
  private
    fStrings: TStrings;
    fTemp: TStrings;
    PROCEDURE SetStrings(Value: TStrings);
  protected
  public
    CONSTRUCTOR Create(AOwner: TComponent); override;
    DESTRUCTOR Destroy; override;

    FUNCTION GetValue(Section, Name: STRING): STRING;  //Get a single value if in name=value format
    FUNCTION GetSection(Section: STRING): TStrings;    //Get Section contents
    FUNCTION GetSectionNames(Section: STRING): TStrings; //Get Section names if in name=value format
    FUNCTION GetSectionValues(Section: STRING): TStrings;//Get Section values if in name=value format
    FUNCTION GetSectionHeadings: TStrings;               //Get all Section headings
  published
    PROPERTY Strings: TStrings read FStrings write SetStrings;
END;

PROCEDURE Register;

IMPLEMENTATION

PROCEDURE Register;
BEGIN
  RegisterComponents('Additional', [TInTex]);
END;

FUNCTION TInTex.GetValue(Section, Name: STRING): STRING;
VAR
  I: integer;
BEGIN
  I := fStrings.IndexOf('[' + Section + ']');
  IF I >= 0 THEN
  BEGIN
    FOR I := I + 1 TO fStrings.Count - 1 DO
    BEGIN
      IF Pos('[', fStrings[I]) = 1 THEN Break;
      IF fStrings.Names[I] = Name THEN
      BEGIN
        Result := fStrings.Values[Name];
        Break;
      END;
    END;
  END
  ELSE
  BEGIN
    RAISE ENotFound.Create('The Section "' + Section + '" was not found.');
    Exit;
  END;

  IF Result = '' THEN
    RAISE ENotFound.Create('The value for "' + Name + '" was not found.');
END;

FUNCTION TInTex.GetSectionNames(Section: STRING): TStrings;
VAR
  I: integer;
BEGIN
  fTemp.Clear;
  I := fStrings.IndexOf('[' + Section + ']');
  IF I >= 0 THEN
  BEGIN
    FOR I := I + 1 TO fStrings.Count - 1 DO
      IF Pos('[', fStrings[I]) = 1 THEN Break ELSE fTemp.Add(fStrings.Names[I]);
    Result := fTemp;
  END
  ELSE
    RAISE ENotFound.Create('The Section "' + Section + '" was not found.');

  IF Result.Count = 0 THEN
    RAISE ENotFound.Create('There are no names in section "' + Section + '".');
END;

FUNCTION TInTex.GetSectionValues(Section: STRING): TStrings;
VAR
  I: Integer;
BEGIN
  fTemp.Clear;
  I := fStrings.IndexOf('[' + Section + ']');
  IF I >= 0 THEN
  BEGIN
    FOR I := I + 1 TO fStrings.Count - 1 DO
      IF Pos('[', fStrings[I]) = 1 THEN Break ELSE fTemp.Add(fStrings.Values[fStrings.Names[I]]);
    Result := fTemp;
  END
  ELSE
    RAISE ENotFound.Create('The Section "' + Section + '" was not found.');

  IF Result.Count = 0 THEN
    RAISE ENotFound.Create('There are no values in section "' + Section + '".');
END;

FUNCTION TInTex.GetSection(Section: STRING): TStrings;
VAR
  I: Integer;
BEGIN
  fTemp.Clear;
  I := fStrings.IndexOf('[' + Section + ']');
  IF I >= 0 THEN
  BEGIN
    FOR I := I + 1 TO fStrings.Count - 1 DO
      IF Pos('[', fStrings[I]) = 1 THEN Break ELSE fTemp.Add(fStrings[I]);
    Result := fTemp;
  END
  ELSE
    RAISE ENotFound.Create('The Section "' + Section + '" was not found.');
END;

FUNCTION TInTex.GetSectionHeadings: TStrings;
VAR
  I: integer;
BEGIN
  fTemp.Clear;
  FOR I := 0 TO fStrings.Count - 1 DO
    IF Pos('[', fStrings[I]) = 1 THEN fTemp.Add(fStrings[I]);
  Result := fTemp;

  IF Result.Count = 0 THEN
    RAISE ENotFound.Create('No section headings were found.');
END;

PROCEDURE TInTex.SetStrings(Value: TStrings);
BEGIN
  IF fStrings <> value THEN
  BEGIN
    fStrings.BeginUpdate;
    fStrings.Assign(Value);
    fStrings.EndUpDate;
  END;
END;


CONSTRUCTOR TInTex.Create(AOwner: TComponent);
BEGIN
  INHERITED Create(AOwner);
  fStrings := TStringList.Create;
  fTemp := TStringList.Create;
END;

DESTRUCTOR TInTex.Destroy;
BEGIN
  fStrings.Free;
  fTemp.Free;
  INHERITED Destroy;
END;

END.

